//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

//@Entity
//@Table(name = "annotation_element", schema="public")
public class AnnotationElement implements Serializable {

	public final static int STATUS_DEFECT = -1;
	public final static int STATUS_OPEN = 0;
	public final static int STATUS_IN_WORK = 1;
	public final static int STATUS_FINISHED = 2;
	public final static int STATUS_REVOKED = 3;

	// TODO Does an AnnotationElement know it's template?

	protected int annotationElementId;
	protected String annotationText;
	protected Date beginOfAnnotation;
	protected Date endOfAnnotation;
	protected int status;
	protected String comment;
	protected AnnotationGroup parent;// ManyToOne
	protected Set<RecordingFile> recordingFiles = new HashSet<RecordingFile>(0);// ManyToMany
	protected String type;
	protected String subtype;

	/** default constructor */
	public AnnotationElement() {
	}

	/** minimal constructor */
	public AnnotationElement(int annotationElementId) {
		this.annotationElementId = annotationElementId;
	}

	/** child constructor */
	public AnnotationElement(AnnotationElement child) {
		/*
		 * As childs will be initialized only in order to update Elements to
		 * Database or send them to ws-client, references do not have to be
		 * cloned. 
		 */
		this();
		merge(child);
	}

	protected void merge(AnnotationElement child) {
		this.annotationElementId = child.annotationElementId;
		this.annotationText = child.annotationText;
		this.beginOfAnnotation = child.beginOfAnnotation;
		this.endOfAnnotation = child.endOfAnnotation;
		this.status = child.status;
		this.comment = child.comment;
		this.parent = child.parent;
		this.recordingFiles = child.recordingFiles;
		this.type = child.type;
		this.subtype = child.subtype;
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationElement(";
		if (annotationElementId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationElementId));
		}
		name = name.concat(":" + type + "|" + subtype + ")");
		return name;
	}

}
