//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name = "annotation_element", schema = "public")
public class AnnotationElementDb extends AnnotationElement {

	/** default constructor */
	public AnnotationElementDb() {
		super();
	}

	public AnnotationElementDb(AnnotationElement source) {
		super(source);
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationElementDb(";
		if (annotationElementId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationElementId));
		}
		name = name.concat(":" + type + "|" + subtype + ")");
		return name;
	}

	public void merge(AnnotationElement changedProperties) {
		if (this.annotationElementId != 0
				&& changedProperties.annotationElementId != this.annotationElementId) {
			throw new IllegalArgumentException(
					"Elements do not have the same Id!");
		}
		super.merge(changedProperties);
	}

	@Id
	@Column(name = "annotation_element_id", unique = true, nullable = false)
	@GeneratedValue(generator = "id_gen")
	public int getAnnotationElementId() {
		return annotationElementId;
	}

	public void setAnnotationElementId(int annotationElementId) {
		this.annotationElementId = annotationElementId;
	}

	@Column(name = "annotation_text", length = 1000)
	public String getAnnotationText() {
		return annotationText;
	}

	public void setAnnotationText(String annotationText) {
		this.annotationText = annotationText;
	}

	@Column(name = "begin_annotation")
	@Temporal(TemporalType.TIMESTAMP)
	public Date getBeginOfAnnotation() {
		return beginOfAnnotation;
	}

	public void setBeginOfAnnotation(Date beginOfAnnotation) {
		this.beginOfAnnotation = beginOfAnnotation;
	}

	@Column(name = "end_annotation")
	@Temporal(TemporalType.TIMESTAMP)
	public Date getEndOfAnnotation() {
		return endOfAnnotation;
	}

	public void setEndOfAnnotation(Date endOfAnnotation) {
		this.endOfAnnotation = endOfAnnotation;
	}

	@Column(name = "status")
	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	@Column(name = "comment", length = 1000)
	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	@ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	// not null? //TODO cascade?
	@JoinColumn(name = "parent_id")
	public AnnotationGroupDb getParent() {
		if (parent == null) {
			return null;
		} else {
			if (parent.getClass() != AnnotationGroupDb.class) {
				parent = new AnnotationGroupDb(parent);
			}
			return (AnnotationGroupDb) parent;
		}
	}

	public void setParent(AnnotationGroupDb parent) {
		this.parent = parent;
	}

	@ManyToMany(cascade = { CascadeType.PERSIST }, fetch = FetchType.EAGER)
	@JoinTable(name = "annotation_element_recording_file", joinColumns = { @JoinColumn(name = "annotation_element_id") }, inverseJoinColumns = { @JoinColumn(name = "recording_file_id") })
	// TODO Find out whether RecordingFile should know it's annotationElements
	public Set<RecordingFile> getRecordingFiles() {
		return recordingFiles;
	}

	public void setRecordingFiles(Set<RecordingFile> recordingFiles) {
		this.recordingFiles = recordingFiles;
	}

	@Column(name = "type", length = 200)
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	@Column(name = "subtype", length = 200)
	public String getSubtype() {
		return subtype;
	}

	public void setSubtype(String subtype) {
		this.subtype = subtype;
	}

}
