//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;

@Entity
@Table(name = "annotation_project")
public class AnnotationProjectDb extends AnnotationProject {

	/** default constructor */
	public AnnotationProjectDb() {
		super();
	}

	/** minimal constructor */
	public AnnotationProjectDb(int annotationProjectId) {
		super(annotationProjectId);
	}

	public AnnotationProjectDb(AnnotationProject source) {
		super(source);
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationProjectDb(";
		if (annotationProjectId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationProjectId));
		}
		name = name.concat(":" + name + ")");
		return name;
	}
	
	public void merge(AnnotationProject changedProperties){
		if(this.annotationProjectId != 0 && changedProperties.annotationProjectId != this.annotationProjectId){
			throw new IllegalArgumentException("Projects do not have the same Id!");
		}
		super.merge(changedProperties);
	}

	@Id
	@Column(name = "annotation_project_id", unique = true, nullable = false)
	@GeneratedValue(generator = "id_gen")
	// @ResourceKey("id")
	public int getAnnotationProjectId() {
		return annotationProjectId;
	}

	public void setAnnotationProjectId(int annotationProjectId) {
		this.annotationProjectId = annotationProjectId;
	}

	@Column(name = "annotation_project_name", length = 200)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "description", length = 1000)
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@ManyToMany
	@JoinTable(name = "admins_of_annotation_projects", joinColumns = { @JoinColumn(name = "annotation_project") }, inverseJoinColumns = { @JoinColumn(name = "admin_account") })
	public Set<Account> getAdmins() {
		return admins;
	}

	public void setAdmins(Set<Account> admins) {
		this.admins = admins;
	}

	/**
	 * Warning: Changes on the returned Set will not affect this Set. Use
	 * setSessions after changing!
	 * 
	 * @return a Copy of the contained Set<AnnotationSession> as
	 *         Set<AnnotationSessionDb>
	 */
	@OneToMany(fetch = FetchType.LAZY, cascade= {CascadeType.REMOVE})
	// no "mappedBy" -> extra Table "annotation_project_annotation_session
	@JoinTable(name = "annotation_sessions_in_project")
	public Set<AnnotationSessionDb> getSessions() {
		Set<AnnotationSessionDb> dbSessions = new HashSet<AnnotationSessionDb>();
		Iterator<AnnotationSession> it = sessions.iterator();
		while (it.hasNext()) {
			AnnotationSession next = it.next();
			if(next.getClass() != AnnotationSessionDb.class){
				next = new AnnotationSessionDb(next);
			}
			dbSessions.add((AnnotationSessionDb) next);
		}
		return dbSessions;
	}

	public void setSessions(Set<AnnotationSessionDb> hashSet) {
		this.sessions.clear();
		this.sessions.addAll(hashSet);
	}

}
